# SPDX-License-Identifier: Apache-2.0

import json
from google.protobuf.json_format import MessageToJson
from clouditor_interface.ids_provider import map_resource_id
from grpc_gen.assessment_pb2 import AssessEvidenceRequest

# Used if user doesn't provide other
_default_resource_type = ["VirtualMachine", "Compute", "Resource"]

def create_resource(name, type, property_list):
    resource = {
        "id": str(map_resource_id(name)),
        "name": str(name),
        "type": type if type is not None else _default_resource_type
    }

    if property_list is not None:
        resource.update(property_list)

    return resource

def create_assess_evidence_request(id, cloud_service_id, tool_id, raw, resource):
    aer = AssessEvidenceRequest()
    aer.evidence.id = str(id)
    aer.evidence.timestamp.GetCurrentTime()
    aer.evidence.timestamp.nanos = 0
    aer.evidence.cloud_service_id = str(cloud_service_id)
    aer.evidence.tool_id = str(tool_id)
    aer.evidence.raw = json.dumps(raw)
    aer.evidence.resource.struct_value.update(resource)

    return aer

def print_evidence(logger, evidence):
    logger.debug(MessageToJson(evidence))
