# SPDX-License-Identifier: Apache-2.0

import json
from google.protobuf.json_format import MessageToDict
from clouditor_interface.orchestrator_interface import OrchestratorInterface
from clouditor_interface.orchestrator_interface_messages import create_list_cloud_services_request, create_register_cloud_service_request
from clouditor_interface.clouditor_authentication import ClouditorAuthentication
import uuid
from log_conf.log_conf import LOGGER
import configparser

oauth_client = ClouditorAuthentication(LOGGER)

oi = OrchestratorInterface(LOGGER)

# Map Cloud Service Names from 'id_maps/cloud_service_name_map.json'
def map_cloud_service_name(name):
    try:
        f = open('id_maps/cloud_service_name_map.json')
        map = json.load(f)
        f.close()
    except:
        map = {}

    if name in map:
        return map[name]
    else:
        return name

# Map Cloud Resource IDs from 'id_maps/resource_id_map.json'
def map_resource_id(name):
    try:
        f = open('id_maps/resource_id_map.json')
        map = json.load(f)
        f.close()
    except:
        map = {}

    if name in map:
        return map[name]
    else:
        return name

# Get tool ID (SERVICE:VERSION format)
def get_tool_id():
    with open('MANIFEST', 'r') as f:
        config_string = '[clouditor]\n' + f.read()

    config = configparser.ConfigParser()
    config.read_string(config_string)

    version = '{}:{}'.format(config.get('clouditor', 'SERVICE'), config.get('clouditor', 'VERSION'))

    return version

# Get dict of all Cloud Services from Orchestrator API
def get_cloud_services():
    listCloudServicesRequest = create_list_cloud_services_request(50, "", "name", True)
    response = oi.list_cloud_services(listCloudServicesRequest, oauth_client.get_token())
    
    response_json = MessageToDict(response)

    return response_json['services']

# Get Cloud Service ID (if it exists) or create a new Cloud Service if it doesn't
def get_cloud_service_id(name):
    cloud_service_map = map_cloud_service_name(name)

    cloud_services = get_cloud_services()

    for service in cloud_services:
        if service['name'] == cloud_service_map["name"]:
            return service['id']

    # Create new Cloud Service
    id = uuid.uuid1()
    crcsr = create_register_cloud_service_request(id, cloud_service_map["name"], cloud_service_map["description"] if "description" in cloud_service_map else None)
    response = oi.register_cloud_service(crcsr, oauth_client.get_token())

    response_dict = MessageToDict(response)
    
    LOGGER.debug("New Cloud Service '" + response_dict['name'] + "' (ID: '" + response_dict['id'] + "') successfully created")

    return response_dict['id']
