# SPDX-License-Identifier: Apache-2.0

from grpc_gen.orchestrator_pb2_grpc import OrchestratorStub
import grpc
import os

LOCAL_CLOUDITOR_DEPLOY = os.environ.get("local_clouditor_deploy").lower() in ('true', '1', 't')
ORCHESTRATOR_HOST = os.environ.get("orchestrator_host")
ORCHESTRATOR_PORT = int(os.environ.get("orchestrator_port"))

class OrchestratorInterface(object):

    def __init__(self, logger):
        if LOCAL_CLOUDITOR_DEPLOY:
            self.channel = grpc.insecure_channel('{}:{}'.format(ORCHESTRATOR_HOST, ORCHESTRATOR_PORT))
        else:
            ssl_metadata = grpc.ssl_channel_credentials()
            self.channel = grpc.secure_channel('{}:{}'.format(ORCHESTRATOR_HOST, ORCHESTRATOR_PORT), ssl_metadata)

        self.stub = OrchestratorStub(self.channel)
        self.logger = logger

    def list_cloud_services(self, listcloudservicesrequest, token):
        try:
            if token is not None:
                metadata = [('authorization', 'Bearer ' + token)]
            else:
                metadata = None

            response = self.stub.ListCloudServices(listcloudservicesrequest, metadata=metadata)
            self.logger.debug('gRPC list cloud services request successfully forwarded')

            return response
        except grpc.RpcError as err:
            self.logger.error(err)
            self.logger.error(err.details())
            self.logger.error('{}, {}'.format(err.code().name, err.code().value))

            self.logger.debug("Unable to acquire cloud services list from the Orchestrator")

            return None

    def register_cloud_service(self, registercloudservicerequest, token):
        try:
            if token is not None:
                metadata = [('authorization', 'Bearer ' + token)]
            else:
                metadata = None

            response = self.stub.RegisterCloudService(registercloudservicerequest, metadata=metadata, timeout=1)
            self.logger.debug('gRPC register cloud service request successfully forwarded')

            return response
        except grpc.RpcError as err:
            self.logger.error(err)
            self.logger.error(err.details())
            self.logger.error('{}, {}'.format(err.code().name, err.code().value))
