# SPDX-License-Identifier: Apache-2.0

import json
import urllib3

class VatClient:
	
	def __init__(self, protocol, ip, port, api_prefix, logger):
		self._protocol = protocol
		self._ip = ip
		self._port = port
		self._api_prefix = api_prefix
		self.logger = logger
	
	def req(self, method, resource, data=None, headers={}, fields=None):
		if self._protocol == "https":
			c = urllib3.HTTPSConnectionPool(host=self._ip, port=self._port, cert_reqs='CERT_NONE', assert_hostname=False)
		else:
			c = urllib3.HTTPConnectionPool(host=self._ip, port=self._port)

		url = "%s://%s:%i%s/%s" % (self._protocol, self._ip, self._port, self._api_prefix, resource)

		try:
			if not fields:
				resp = c.request(method, url, headers=headers, body=data)
			else:
				resp = c.request(method, url, headers=headers, fields=fields)

		except (TimeoutError, urllib3.exceptions.NewConnectionError, 
        		urllib3.exceptions.MaxRetryError,
				urllib3.exceptions.ConnectTimeoutError,
				urllib3.exceptions.TimeoutError) as err:
			self.logger.exception("VAT not available: " + str(err), exc_info=False)
			raise
		else:
			if resp.status in range(200, 300):
				return json.loads(resp.data)
			else:
				self.logger.exception("VAT not available: got response code " + str(resp.status), exc_info=False)
				raise
